/* 
 *
 * $Id: k3bmonkeyio.cpp,v 1.1.1.1 2004/03/09 21:34:16 trueg Exp $
 * Copyright (C) 2003 Sebastian Trueg <trueg@k3b.org>
 *
 * This file is part of the K3b project.
 * Copyright (C) 1998-2003 Sebastian Trueg <trueg@k3b.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */

#include "k3bmonkeyio.h"

#include <qfile.h>
#include <qdatastream.h>

#include <kdebug.h>

#include <unistd.h>
#include <sys/types.h>



class K3bMonkeyIO::Private
{
public:
  Private() {
  }

  QFile file;
};



K3bMonkeyIO::K3bMonkeyIO() 
  : CIO()
{
  d = new Private();
}


K3bMonkeyIO::~K3bMonkeyIO()
{
  delete d;
}


int K3bMonkeyIO::Open(const wchar_t * pName)
{
  return Open( QString::fromUtf8( (char*)GetUTF8FromUTF16(pName) ) );
}


int K3bMonkeyIO::Open(const QString& pName)
{
  Close();

  d->file.setName( pName );

  if( d->file.open( IO_ReadWrite ) )
    return 0;
  else
    return -1;
}


int K3bMonkeyIO::Close()
{
  if( d->file.isOpen() ) {
    d->file.close();
    return 0;
  }
  else
    return -1;
}


int K3bMonkeyIO::Read(void * pBuffer, unsigned int nBytesToRead, unsigned int * pBytesRead)
{
  Q_LONG read = d->file.readBlock( static_cast<char*>(pBuffer), nBytesToRead );
  if( read < 0 )
    return -1;
  else {
    *pBytesRead = read;
    return 0;
  }
}


int K3bMonkeyIO::Write(const void * pBuffer, unsigned int nBytesToWrite, unsigned int * pBytesWritten)
{
  Q_LONG written =  d->file.writeBlock( static_cast<const char*>(pBuffer), nBytesToWrite );
  if( written < 0 )
    return -1;
  else {
    *pBytesWritten = written;
    return 0;
  }
}


int K3bMonkeyIO::Seek(int nDistance, unsigned int nMoveMode)
{
  QFile::Offset pos = (QFile::Offset)0;

  switch( nMoveMode ) {
  case SEEK_CUR:
    pos = d->file.at() + nDistance;
    break;
  case SEEK_END:
    pos = d->file.size() - nDistance;
    break;
  case SEEK_SET:
  default:
    pos = nDistance;
    break;
  }

  if( d->file.at( pos ) )
    return 0;
  else
    return -1;
}


int K3bMonkeyIO::Create(const wchar_t * pName)
{
  return Create( GetANSIFromUTF16(pName) );
}


int K3bMonkeyIO::Create(const char * pName)
{
  return Open( pName );
}


int K3bMonkeyIO::Delete()
{
  if( d->file.remove() )
    return 0;
  else
    return -1;
}


int K3bMonkeyIO::SetEOF()
{
  // this is not directly supported by QFile so we need to do it manually
  int fd = d->file.handle();
  if( fd > 0 ) {
    return ::ftruncate( fd, d->file.at() );
  }
  else
    return -1;
}


int K3bMonkeyIO::GetPosition()
{
  return d->file.at();
}


int K3bMonkeyIO::GetSize()
{
  return d->file.size();
}


int K3bMonkeyIO::GetName(wchar_t * pBuffer)
{
  char *ptemp;
  ptemp = new char[wcslen(pBuffer)];
  
  GetName(ptemp);
  wcscpy(pBuffer, GetUTF16FromANSI(ptemp));
  
  delete [] ptemp;
  
  return 0;
}


int K3bMonkeyIO::GetName(char * pBuffer)
{
  qstrcpy( pBuffer, QFile::encodeName(d->file.name()).data() );
  return 0;
}

